#include "HandleView.h"

#include <Application.h>
#include <cstdio>
#include <Screen.h>
#include <MenuItem.h>
#include <PopUpMenu.h>
#include <Alert.h>
#include <Dragger.h>
#include <Roster.h>

#include "defines.h"
#include "PanelWindow.h"

// from private_roster.h (or something like that, stolen from Deskbar src :)
const uint32 CMD_SHUTDOWN_SYSTEM = 301;
const uint32 CMD_REBOOT_SYSTEM = 302;
const uint32 CMD_SUSPEND_SYSTEM = 304;

HandleView::HandleView()
:	BView(
		BRect(0,0,10,35),
		"handle",
		B_FOLLOW_NONE,
		B_WILL_DRAW|B_PULSE_NEEDED|B_FULL_UPDATE_ON_RESIZE
	),
	m_mouse_down( false ),
	m_moving_panel( false )
{
	SetViewColor( 255,203,0,0 );
}

HandleView::HandleView( BMessage * msg )
:	BView( msg ),
	m_mouse_down( false ),
	m_moving_panel( false )
{
	SetFlags( Flags() | B_FULL_UPDATE_ON_RESIZE );
}

HandleView::~HandleView()
{
}

BArchivable *
HandleView::Instantiate( BMessage * msg )
{
	if ( !validate_instantiation( msg, "HandleView" ) )
		return NULL;
	return new HandleView(msg);
}

status_t
HandleView::Archive( BMessage * msg, bool deep ) const
{
	return BView::Archive(msg,deep);
}
/*
void
HandleView::Pulse()
{
	Window()->PostMessage( PANEL_MODIFY_SIZE );
}
*/
void
HandleView::MouseDown( BPoint pos )
{
	SetMouseEventMask( B_POINTER_EVENTS, B_LOCK_WINDOW_FOCUS );
	Window()->CurrentMessage()->FindInt32("buttons",&m_buttons);
	
	m_mouse_down = true;
	
	Invalidate();
}

void
HandleView::MouseUp( BPoint pos )
{
	if ( !m_mouse_down )
		return;
	
	if ( m_buttons & B_SECONDARY_MOUSE_BUTTON )
	{
		if ( !m_moving_panel )
		{ // show pop-up
			ShowPopup( ConvertToScreen( pos ) );
		}
	} else
	{ // primary mouse button
		if ( Bounds().Contains( pos ) )
			Window()->PostMessage( PANEL_TOGGLE_OPEN );
	}
	
	m_mouse_down = false;
	m_moving_panel = false;

	Invalidate();
}

void
HandleView::MouseMoved( BPoint point, uint32 transit, const BMessage * )
{
	// for 'expand on mouse over'
	if ( transit == B_ENTERED_VIEW && !m_mouse_down )
	{ // enter view
		Window()->PostMessage( PANEL_MOUSE_ENTER );
	}
	
	if ( ( transit == B_EXITED_VIEW || transit == B_OUTSIDE_VIEW ) && !m_mouse_down )
	{ // exit view (and window)
		if ( !Window()->Frame().Contains( ConvertToScreen(point) ) )
			Window()->PostMessage( PANEL_MOUSE_EXIT );
	}
	
	int32 buttons = 0;
	Window()->CurrentMessage()->FindInt32("buttons",&buttons);
	
	if ( m_mouse_down && (buttons & B_SECONDARY_MOUSE_BUTTON) )
	{ // secondary mouse button pressed
		BPoint scr_pos = ConvertToScreen(point);
		if ( !Window()->Frame().Contains( scr_pos ) || m_moving_panel )
		{ // move window
			m_moving_panel = true;
			
			BScreen screen( Window() );
			
			const float CORNER = 10;
			
			if ( scr_pos.y == 0 )
			{
				if ( scr_pos.x > CORNER && scr_pos.x < (screen.Frame().right - CORNER ) )
				{
					BMessage msg( PANEL_SET_EDGE );
					msg.AddInt8( "edge", PanelWindow::TOP );
					Window()->PostMessage( &msg );
				}
			}
			if ( scr_pos.x == 0 )
			{
				if ( scr_pos.y > CORNER && scr_pos.y < (screen.Frame().bottom - CORNER ) )
				{
					BMessage msg( PANEL_SET_EDGE );
					msg.AddInt8( "edge", PanelWindow::LEFT );
					Window()->PostMessage( &msg );
				}
			}
			if ( scr_pos.x == screen.Frame().right )
			{
				if ( scr_pos.y > CORNER && scr_pos.y < (screen.Frame().bottom - CORNER ) )
				{
					BMessage msg( PANEL_SET_EDGE );
					msg.AddInt8( "edge", PanelWindow::RIGHT );
					Window()->PostMessage( &msg );
				}
			}
			if ( scr_pos.y == screen.Frame().bottom )
			{
				if ( scr_pos.x > CORNER && scr_pos.x < (screen.Frame().right - CORNER ) )
				{
					BMessage msg( PANEL_SET_EDGE );
					msg.AddInt8( "edge", PanelWindow::BOTTOM );
					Window()->PostMessage( &msg );
				}
			}
			
			BMessage msg(PANEL_MOVE_TO );
			msg.AddPoint( "pos", scr_pos );
			Window()->PostMessage( &msg );
		}
	}
}

void
HandleView::Draw( BRect )
{
	rgb_color light  = ViewColor();
	rgb_color bright = ViewColor();
	rgb_color dark   = ViewColor();
	
	// set up colors
	if ( bright.red < 175 )   bright.red += 80;   else bright.red = 255;
	if ( bright.green < 175 ) bright.green += 80; else bright.green = 255;
	if ( bright.blue < 175 )  bright.blue += 80;  else bright.blue = 255;
	
	if ( dark.red > 79 )   dark.red -= 80;   else dark.red = 0;
	if ( dark.green > 79 ) dark.green -= 80; else dark.green = 0;
	if ( dark.blue > 79 )  dark.blue -= 80;  else dark.blue = 0;
	
	if ( m_mouse_down )
	{
		light	= tint_color( light, B_HIGHLIGHT_BACKGROUND_TINT );
		bright	= tint_color( bright, B_HIGHLIGHT_BACKGROUND_TINT );
		dark	= tint_color( dark, B_HIGHLIGHT_BACKGROUND_TINT );
	}
	
	BRect bounds = Bounds();
	
	// border
	MovePenTo( bounds.RightTop() );
	
	SetHighColor( bright );		
	StrokeLine( bounds.LeftTop() );
	StrokeLine( bounds.LeftBottom() );
	
	SetHighColor( dark );
	StrokeLine( bounds.RightBottom() );
	StrokeLine( bounds.RightTop() );
	
	// interior
	BRect temp = bounds; temp.InsetBy(1,1);
	SetHighColor( light );		FillRect( temp );
	
	float minx = 3;
	float maxx = bounds.right-2;
	float miny = 2;
	float maxy = bounds.bottom-1;
	
	if ( bounds.Width() > bounds.Height() )
	{
		minx = 2;
		maxx = bounds.right-1;
		miny = 3;
		maxy = bounds.bottom-2;
	}
	
	// grip-friendly surface look
	SetHighColor( bright );
	for ( float x=minx; x<maxx; x+=3 )
	{
		for ( float y=miny; y<maxy; y+=3 )
		{
			BPoint pos(x,y);
			StrokeLine( pos,pos );
		}
	}

	SetHighColor( dark );
	for ( float x=minx+1; x<maxx; x+=3 )
	{
		for ( float y=miny+1; y<maxy; y+=3 )
		{
			BPoint pos(x,y);
			StrokeLine( pos,pos );
		}
	}
}


void
HandleView::MessageReceived( BMessage * msg )
{
	switch ( msg->what )
	{
		case 'Sfly':
		{ // set floating on
			BMessage win_msg( PANEL_SET_FLOATING );
			win_msg.AddBool("floating_window",true);
			Window()->PostMessage( &win_msg );
		}	break;
		case 'Sfln':
		{ // set floating off
			BMessage win_msg( PANEL_SET_FLOATING );
			win_msg.AddBool("floating_window",false);
			Window()->PostMessage( &win_msg );
		}	break;

		case 'THno':
		{ // set not thin handle
			BMessage win_msg( PANEL_SET_THIN_HANDLE );
			win_msg.AddBool("thin_handle",false);
			Window()->PostMessage( &win_msg );
		}	break;
		case 'THth':
		{ // set thin handle
			BMessage win_msg( PANEL_SET_THIN_HANDLE );
			win_msg.AddBool("thin_handle",true);
			Window()->PostMessage( &win_msg );
		}	break;

		case 'Sico':
		{ // set small icons
			BMessage win_msg( PANEL_SET_ICON_SIZE );
			win_msg.AddBool("small_icons",true);
			Window()->PostMessage( &win_msg );
		}	break;
		case 'Lico':
		{ // set large icons
			BMessage win_msg( PANEL_SET_ICON_SIZE );
			win_msg.AddBool("small_icons",false);
			Window()->PostMessage( &win_msg );
		}	break;

		case 'DATA':
		{ // file drop
			msg->what = PANEL_ADD_ICON;
			Window()->PostMessage(msg);
		}	break;
		case 'Quit':
		{
			printf("Quit requested\n");
			be_app->PostMessage( B_QUIT_REQUESTED );
		}
			break;
		case 'PSTE':
		{ // color drop
			rgb_color * col;
			ssize_t sz;
			if (msg->FindData("RGBColor",B_RGB_COLOR_TYPE, (const void**)&col, &sz) == B_OK)
			{
				SetViewColor( *col );
				Invalidate();
			}
		}
			break;
		case 'Npan':
		{ // new panel
			be_app->PostMessage( msg );
		}	break;
		case 'Rpan':
		{ // remove panel
			BAlert * alert = new BAlert(
				"title",
				__("Are you sure you want to remove this panel?"),
				__("Yes"),
				__("No")
			);
			
			if ( alert->Go() != 0 )
				break;
			
			msg->AddPointer("panel", Window());
			be_app->PostMessage( msg );
		}	break;
		case 'SCsl':
		{ // Set scroll rate, SLOW
			BMessage win_msg( PANEL_SET_SCROLL_RATE );
			win_msg.AddInt64("scroll_rate", 500000 );
			Window()->PostMessage( &win_msg );
		}	break;
		case 'SCno':
		{ // Set scroll rate, NORMAL
			BMessage win_msg( PANEL_SET_SCROLL_RATE );
			win_msg.AddInt64("scroll_rate", 100000 );
			Window()->PostMessage( &win_msg );
		}	break;
		case 'SCfa':
		{ // Set scroll rate, FAST
			BMessage win_msg( PANEL_SET_SCROLL_RATE );
			win_msg.AddInt64("scroll_rate", 20000 );
			Window()->PostMessage( &win_msg );
		}	break;
		case 'SCin':
		{ // Set scroll rate, INSTANT
			BMessage win_msg( PANEL_SET_SCROLL_RATE );
			win_msg.AddInt64("scroll_rate", 0 );
			Window()->PostMessage( &win_msg );
		}	break;
		case 'Semo':
		{ // Set expand on mouse .. ON
			BMessage win_msg(PANEL_SET_EXPAND_ON_MOUSE_OVER);
			win_msg.AddBool("expand_on_mouse_over",true);
			Window()->PostMessage( &win_msg );
		}	break;
		case 'Semf':
		{ // Set expand on mouse .. OFF
			BMessage win_msg(PANEL_SET_EXPAND_ON_MOUSE_OVER);
			win_msg.AddBool("expand_on_mouse_over",false);
			Window()->PostMessage( &win_msg );
		}	break;
		case 'REsh':
		{ // Show replicants
			BDragger::ShowAllDraggers();
		}	break;
		case 'REhi':
		{ // Hide replicants
			BDragger::HideAllDraggers();
		}	break;
		case 'reTR':
		case 'reDB':
		case 'reTE':
		{ // rescue menu
			BRoster roster;
			if (msg->what == 'reTR')
				roster.Launch("application/x-vnd.Be-TRAK");
			if (msg->what == 'reDB')
				roster.Launch("application/x-vnd.Be-TSKB");
			if (msg->what == 'reTE')
				roster.Launch("application/x-vnd.Be-SHEL");
		}
			break;
		case CMD_SHUTDOWN_SYSTEM:
		case CMD_REBOOT_SYSTEM:
		{ // forward to Roster
			BMessenger msgr("application/x-vnd.Be-ROST");
			msgr.SendMessage( msg );
		}
			break;
		default:
			BView::MessageReceived(msg);
	}
}

void
HandleView::ShowPopup( BPoint pos )
{
	PanelWindow * panel = (PanelWindow*)Window();
	
	BPopUpMenu * menu = new BPopUpMenu(
		"context",
		false, // radio
		false // label from marked
	);
	
	BMenu * sub_menu;
	
	sub_menu = new BMenu( __("Size") );
		// check if large
		bool large_icons = false;
		if ( Bounds().Width() > Bounds().Height() )
		{
			if ( Bounds().Width() > 20 )
				large_icons = true;
		} else
		{
			if ( Bounds().Height() > 20 )
				large_icons = true;
		}
		// create items
		sub_menu->AddItem( new BMenuItem(__("Large"), new BMessage('Lico')) );
		sub_menu->AddItem( new BMenuItem(__("Small"), new BMessage('Sico')) );
		// set mark
		if ( large_icons )
			sub_menu->FindItem(__("Large"))->SetMarked(true);
		else
			sub_menu->FindItem(__("Small"))->SetMarked(true);
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	sub_menu = new BMenu( __("Thickness"));
		// check if thick
		bool thin_handle = false;
		if ( Bounds().Width() == 1 || Bounds().Height() == 1 )
		{
			thin_handle = true;
		}
		// create items
		sub_menu->AddItem( new BMenuItem(__("Normal"), new BMessage('THno')) );
		sub_menu->AddItem( new BMenuItem(__("Thin"), new BMessage('THth')) );
		// set mark
		if ( !thin_handle )
			sub_menu->FindItem(__("Normal"))->SetMarked(true);
		else
			sub_menu->FindItem(__("Thin"))->SetMarked(true);
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	sub_menu = new BMenu( __("Scroll speed") );
		sub_menu->AddItem( new BMenuItem(__("Slow"), new BMessage('SCsl')) );
		sub_menu->AddItem( new BMenuItem(__("Normal"), new BMessage('SCno')) );
		sub_menu->AddItem( new BMenuItem(__("Fast"), new BMessage('SCfa')) );
		sub_menu->AddItem( new BMenuItem(__("Instant"), new BMessage('SCin')) );
		// check mark
		if ( panel->AnimationDelay() == 500000 )
			sub_menu->FindItem(__("Slow"))->SetMarked(true);
		if ( panel->AnimationDelay() == 100000 )
			sub_menu->FindItem(__("Normal"))->SetMarked(true);
		if ( panel->AnimationDelay() == 20000 )
			sub_menu->FindItem(__("Fast"))->SetMarked(true);
		if ( panel->AnimationDelay() == 0 )
			sub_menu->FindItem(__("Instant"))->SetMarked(true);
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	sub_menu = new BMenu( __("Expand on mouse-over"));
		sub_menu->AddItem( new BMenuItem(__("On"), new BMessage('Semo')) );
		sub_menu->AddItem( new BMenuItem(__("Off"), new BMessage('Semf')) );
		// set mark
		if ( panel->IsExpandOnMouseOver() )
			sub_menu->FindItem(__("On"))->SetMarked(true);
		else
			sub_menu->FindItem(__("Off"))->SetMarked(true);
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	sub_menu = new BMenu(__("Floating"));
		sub_menu->AddItem( new BMenuItem(__("On"), new BMessage('Sfly')) );
		sub_menu->AddItem( new BMenuItem(__("Off"), new BMessage('Sfln')) );
		// set mark
		if ( Window()->Feel() == B_FLOATING_ALL_WINDOW_FEEL )
			sub_menu->FindItem(__("On"))->SetMarked(true);
		else
			sub_menu->FindItem(__("Off"))->SetMarked(true);
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	
	sub_menu = new BMenu( __("Panel"));
		sub_menu->AddItem( new BMenuItem(__("New"), new BMessage('Npan')) );
		sub_menu->AddItem( new BMenuItem(__("Remove this"), new BMessage('Rpan')) );
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	menu->AddSeparatorItem();
	
	sub_menu = new BMenu( __("Replicants"));
		sub_menu->AddItem( new BMenuItem(__("Show"), new BMessage('REsh')) );
		sub_menu->AddItem( new BMenuItem(__("Hide"), new BMessage('REhi')) );
		// set mark
		if ( BDragger::AreDraggersDrawn() )
			sub_menu->FindItem(__("Show"))->SetMarked(true);
		else
			sub_menu->FindItem(__("Hide"))->SetMarked(true);
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	sub_menu = new BMenu( __("Rescue"));
		sub_menu->AddItem( new BMenuItem(__("Tracker"), new BMessage('reTR')) );
		sub_menu->AddItem( new BMenuItem(__("Deskbar"), new BMessage('reDB')) );
		sub_menu->AddItem( new BMenuItem(__("Terminal"), new BMessage('reTE')) );
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	sub_menu = new BMenu( __("Shutdown"));
		sub_menu->AddItem( new BMenuItem(__("Reboot"), new BMessage(CMD_REBOOT_SYSTEM)) );
		sub_menu->AddItem( new BMenuItem(__("Power off"), new BMessage(CMD_SHUTDOWN_SYSTEM)) );
		// set target
		sub_menu->SetTargetForItems( this );
	menu->AddItem( sub_menu );
	
	#ifdef USE_SPLOCALE
	menu->AddSeparatorItem();
	
	((SpLocaleApp*)be_app)->AddToFileMenu(menu, false, false, false );
	#endif
	
	menu->AddSeparatorItem();
	
	menu->AddItem( new BMenuItem(__("Quit"), new BMessage('Quit')) );
	
	menu->SetTargetForItems( this );
	
	menu->Go(
		pos,
		true,	// delivers msg
		true	// open anyway
	);
		
	delete menu;
}
